package model

import (
	"time"

	"go.mongodb.org/mongo-driver/bson/primitive"
)

// 生产计划
type ProductPlan struct {
	Id   primitive.ObjectID `bson:"_id,omitempty" json:"_id"`
	Name string             `bson:"name,omitempty" json:"name"`
	// 包装 待生产的产品
	Pack       *Pack  `bson:"pack,omitempty" json:"pack"`
	Thumbnail  string `bson:"thumbnail,omitempty" json:"thumbnail"`
	CreateUser string `bson:"createUser,omitempty" json:"createUser"`

	//生产数量
	Total int `bson:"total,omitempty" json:"total"`

	//状态
	Status string `bson:"status,omitempty" json:"status"`

	//总价
	TotalPrice *float64 `bson:"totalPrice,omitempty" json:"totalPrice"`

	UpdateTime time.Time `bson:"updteTime,omitempty" json:"updateTime"`

	CreateTime time.Time `bson:"createTime,omitempty" json:"createTime"`
}

// 包装
type Pack struct {
	Id        primitive.ObjectID `bson:"_id,omitempty" json:"_id"`
	Name      string             `bson:"name" json:"name"`
	Thumbnail string             `bson:"thumbnail" json:"thumbnail"`
	// 部件数量
	CompCounts int `bson:"compCounts" json:"compCounts"`
	// 设计师
	Designer string `bson:"designer" json:"designer"`
	// 组成包装的部件
	Components []*PackComponent `bson:"components" json:"components"`
	CreateTime time.Time        `bson:"createTime" json:"createTime"`
	UpdateTime time.Time        `bson:"updateTime" json:"updateTime"`
}

type PackComponent struct {
	Id        string `bson:"id" json:"id"`
	Name      string `bson:"name" json:"name"`
	Thumbnail string `bson:"thumbnail" json:"thumbnail"`
	//数量
	Count int `bson:"count" json:"count"`

	//刀版图
	Uv string `bson:"uv" json:"uv"`

	//部件尺寸
	UvSize string `bson:"uvSize" json:"uvSize"`

	//所有材料
	Stages []*ComponentStage `bson:"stages" json:"stages"`

	Remark string `bson:"remark" json:"remark"`

	//部件总价
	TotalPrice float64 `bson:"totalPrice" json:"totalPrice"`
}

type ComponentStage struct {
	Id     string             `bson:"id" json:"id"`
	TypeId primitive.ObjectID `bson:"typeId" json:"typeId"`

	// 下单单价
	OrderPrice float64 `bson:"orderPrice" json:"orderPrice"`

	// 下单数量
	OrderCount int `bson:"orderCount" json:"orderCount"`

	// 是否固定价格 如果是:为预算总价，不是：为实际总价
	IsFix *bool `bson:"isFix" json:"isFix"`

	//实际价格 OrderPrice*ConfirmCount
	//预算价格 OrderPrice*OrderCount

	//确认收货数量
	ConfirmCount int `bson:"confirmCount" json:"confirmCount"`

	// 供应相关信息
	// 交货时间
	DeliveryTime time.Time `bson:"deliveryTime" json:"deliveryTime"`

	// 供应商信息
	SupplierInfo *Supplier `bson:"supplierInfo" json:"supplierInfo"`

	BatchCount      *float64 `bson:"batchCount" json:"batchCount"`           //拼版数量
	BatchSizeWidth  int      `bson:"batchSizeWidth" json:"batchSizeWidth"`   //平板尺寸
	BatchSizeHeight int      `bson:"batchSizeHeight" json:"batchSizeHeight"` //平板尺寸

	Remark     string    `bson:"remark" json:"remark"`     //备注
	Size       string    `bson:"size" json:"size"`         //工艺尺寸
	Name       string    `bson:"name" json:"name"`         //名称
	Category   string    `bson:"category" json:"category"` //分类
	Price      float64   `bson:"price" json:"price"`       //单价
	Unit       string    `bson:"unit" json:"unit"`         //单位
	Norm       string    `bson:"norm" json:"norm"`         // 规格
	CreateTime time.Time `bson:"createTime" json:"createTime"`
	UpdateTime time.Time `bson:"updateTime" json:"updateTime"`
	Type       int       `bson:"type" json:"type"`         //1-材料 2-工艺 3-成品
	Group      string    `bson:"group" json:"group"`       //工序合标记，相同Group的数据合并成一个单据
	BillId     string    `bson:"billId" json:"billId"`     //订单Id
	BillType   int       `bson:"billType" json:"billType"` //订单type // 1,2,3
	Sort       int       `bson:"sort" json:"sort"`
}
